/*

	PCCON.C

	User console BIOS functions for IBM-PC compatible machines.
	Compatible with Microsoft C Version 5.0.

	Copyright (c) 1990 Lindley Systems

	4257 Berwick Place
	Woodbridge, VA 22192-5119
	(703) 590-8890

*/

#include <stdio.h>
#include <dos.h>
#include "keys.h"

#define VIDEOROWS 0x00400084L
#define FONTHEIGHT 0x00400085L

/* BIOS interrupts */
#define VIDEO_IO	0x10
/* video routine numbers */
/* (placed in register AH before a BIOS interrupt 10H */
#define GET_STATE	15

/* Prototypes */
int  system_type();
void init_keys();
void exit_keys();
int  gtkey();
void _wchar(unsigned char c);
void wchar (unsigned char c);
void wstr(unsigned char *str);
void wstr0(unsigned char *str);
void color(int color_byte);
void box(int y1, int x1, int y2, int x2, int dflag);
void put_box_char(int row,int col,unsigned char ch);
void clear_eol();
void erase (int min_row, int min_col, int max_row, int max_col);
void scroll(int top_row,int left_col,int bot_row,int right_col,int nlines);
void cls ();
void setcur(int to_row, int to_col) ;
int  getcur();
void cursize (int kind);
int  getpage();
int  video_mode();
int  set_rows(int num_rows);

static char far *vid_mem;
static int bpr, col, cols, row, vmode;
int rows;

#define VIDEO_INT    0x10
#define SET_VMODE    0x0
#define CUR_SIZE     0x1
#define SET_POSN     0x2
#define READ_POSN    0x3
#define SCROLL_UP    0x6
#define SCROLL_DOWN  0x7
#define WRITE_CHR    0x9
#define WRITE_TTY    0xe
#define GET_VMODE    0xf
#define SET_FONT     0x11

#define BDOS_IN      7

/* special_keymap remaps IBM special keys (NUL followed by code) to Z-100 keys */

unsigned char special_keymap[]=
       {0,0,0,0,0,0,0,0,0,0,					/*   0....9 */
	0,0,0,0,0,SH_TAB,0,0,0,0,				/*  10...19 */
	0,0,0,0,0,0,0,0,0,0,					/*  20...29 */
	0,0,0,0,0,0,0,0,0,0,					/*  30...39 */
	0,0,0,0,0,0,0,0,0,0,					/*  40...49 */
	0,0,0,0,0,0,0,0,0,F1,					/*  50...59 */
	F2,F3,F4,F5,F6,F7,F8,F9,F10,0,				/*  60...69 */
	0,HOME,UP,PGUP,0,LEFT,0,RIGHT,0,END,			/*  70...79 */
	DOWN,PGDN,INS,DEL,SF1,SF2,SF3,SF4,SF5,SF6,		/*  80...89 */
	SF7,SF8,SF9,SF10,CF1,CF2,CF3,CF4,CF5,CF6,		/*  90...99 */
	CF7,CF8,CF9,CF10,AF1,AF2,AF3,AF4,AF5,AF6,		/* 100..109 */
	AF7,AF8,AF9,AF10,0,CTL_LEFT,CTL_RIGHT,SK1,SK3,SK7,	/* 110..119 */
	0,0,0,0,0,0,0,0,0,0,					/* 120..129 */
	SKDASH,SKDOT,SK9,F11,F12,SF11,SF12,CF11,CF12,AF11,	/* 130..139 */
	AF12,0,0,0,0,0,0,0,0,0};				/* 140..149 */

/*	Color attributes. Characters 1..4 use attributes 0..3 */

int cur_attr;
int color_attribs[4]={0x70,0x07,0x7F,0x0F};

static union  REGS Regs;

int  system_type()
{	return(0);		/* Indicate PC */
}

static void video_int(int function,int modifier)
{	Regs.h.ah = function;
	Regs.h.al = modifier;
	int86(VIDEO_INT,&Regs,&Regs);
}

/* gtkey ------------------------------------------------------
   Return a key from the keyboard and map control characters */

int gtkey()
{
    register int c;
 
    switch(c=bdos(BDOS_IN,0,0) & 0xff)
      { case 0:		/* Special function key */
	  return((int)special_keymap[bdos(BDOS_IN,0,0) & 0xff]);
	case '\r':
	  return('\n');
	case '\n':
	  return('\r');
	default:
	  return(c);
      }
}

/* set_rows----------------------------------------------------
   Sets the number of displayed rows on an EGA or VGA display */

int set_rows(int num_rows)
{
    int vmode;

	video_int(SET_VMODE,3);		/* Mode 3 -- alphanumeric */

	/* 0 means CGA or MDA -- do nothing. */
	if ((int)(*(char far *)VIDEOROWS) == 0)
	  return(25);

	switch(num_rows)
	  { case 25:
		vmode = -1;			/* No further action! */
		break;
	    case 28:
		vmode = 0x11;			/* 8x14. (Really 25 rows on EGA) */
		break;
	    default:				/* 43, 50, and all others: */
		vmode = 0x12;			/* 8x8 font */
		break;
	  }

	if (vmode > 0)
	  { Regs.x.bx=0;
	    video_int(SET_FONT,vmode);		/* Set font to given height */
	  }

	rows = (int)(*(char far *)VIDEOROWS);
	return(rows+1);
}

/* video_mode--------------------------------------------------
   Returns the current display mode.

   Sets rows and cols to current display size. (In 80x25, rows=24, cols=79)
   (Error if video_mode == 0xFF)
*/

int video_mode()
{
	rows = (int)(*(char far *)VIDEOROWS);
	if (rows == 0)
	  rows = 24;

	video_int(GET_VMODE,0xFF);	/* Set to illegal value */
	cols = Regs.h.ah - 1;		/* Last Column in display (79 usually) */

	return (int) Regs.h.al;
}

/* key_delay -----------------------------

	Delay a given number of 100ths of seconds, up to 60, or
	until a key is hit.

*/

static int getclk()
{
#ifdef __TURBOC__
  struct time tim_SysTime;

  gettime(&tim_SysTime);
  return((int)(tim_SysTime.ti_sec)*100+tim_SysTime.ti_hund);
#else
  struct dostime_t time;

  _dos_gettime(&time);
  return((int)(time.second)*100+time.hsecond);
#endif
}

void key_delay (int nhunds)
{
  register int waitfor;

  waitfor = getclk()+nhunds;
  if (waitfor >= 6000)	/* Overflowed seconds into minutes */
    waitfor -= 6000;
	/* If we went 'backwards' to the beginning of the next minute,
	   wait for this minute to end... */
  while (getclk() > waitfor && !kbhit())
	;
	/* Now wait until proper moment in the current second */
  while (getclk() < waitfor && !kbhit())
	;
}

/* init_keys ---------------------------------------------------------

   Provided in the Z-100 version to set up the keyboard,
   this actually initializes the display adapter segment number.
*/

void init_keys()
{
    vmode = video_mode();
    if (vmode == 0xFF)		/* Whoa!! Must be Z-100 */
      { fprintf(stderr,"Unknown Video Mode.");
	exit(9);
      }
    bpr = (cols+1)*2;		/* Bytes per Row */
    vid_mem = (vmode == 7) ? (char far *)0xB0000000L : (char far *)0xB8000000L;
    row = col = 0;
    color(color_attribs[1]);
}

/* exit_keys ---------------------------------------------------------
	Restores the screen color to black & white.
	Called just before the application exits.
*/

void exit_keys()

{
	color(0x07);
        cursize(1);
}

/* color -----------------------------------------------------
   Sets the character color attributes

	color_byte = 0bbbffff
			 ^^^^-- 0..15, Foreground
		      ^^^------ 0...7, Background
*/

void color(int color_byte)
{
	cur_attr = color_byte;
}

/* getpage -----------------------------------------------------
   Returns the currently active display page */

int getpage()
{
    video_int(GET_VMODE,0);
    return (int) Regs.h.bh;
}

/* cursize -----------------------------------------------------
   Sets the height of the cursor */

void cursize (int kind)
{
    int top_line, bot_line;

    bot_line = (vmode == 7)?12:7;
	/* 12 scan lines in monochrome, 7 in color */
    switch (kind)
      { case 0:	/* No cursor */
	  top_line = 0x20;
	  break;
	case 1:	/* Underline */
	  top_line = bot_line - 1;
	  break;
	default: /* Block */
	  top_line = 0;
      }

    Regs.h.ch=top_line;
    Regs.h.cl=bot_line;
    video_int(CUR_SIZE,0);
}

/* scur --------------------------------------------------------
   Set cursor to the given position.
*/
static void scur(int rw, int cl, int pagenum)
{
    Regs.h.dh=rw;
    Regs.h.dl=cl;
    Regs.h.bh=pagenum;
    video_int(SET_POSN,0);
}

/* gcur --------------------------------------------------------
   Gets the current cursor position */

static int gcur(int pagenum)
{
    Regs.h.bh=pagenum;
    video_int(READ_POSN,0);
    return (Regs.x.dx);
}

/* setcur ------------------------------------------------------
   Sets the cursor to the given position */

void setcur(int to_row, int to_col) 

{   scur(to_row,to_col,getpage());
    row = to_row; col = to_col;
}

/* cls ---------------------------------------------------------
   Clears the console screen */

void cls ()

{   erase(0,0,rows,cols);
    setcur(0,0);
}

/* getcur ------------------------------------------------------
   Gets the current cursor position */

int getcur()
{
   return gcur(getpage());
}

/* _wchar -------------------------------------------------------
	Writes one character to screen.
	Does Not move the cursor, though it does update the
	current row and column variables.
*/

void _wchar(unsigned char c)
{   char far *v;

    if (c<16)
      switch (c)
      {	case '\01':
	case '\02':
	case '\03':
	case '\04':
	    color(color_attribs[c-1]);
	    break;
	case BEL:  /*  Bell */
	    Regs.h.bl=0;
	    video_int(WRITE_TTY,7);	/* beep! */
	    break;
	case TAB:
	    col = (col+8)&0xf8;
	    break;
	case BS:  /* Backspace */
	    if (col > 0) --col;
	    break;
	case NL:  /* Line feed */
	    if (row < rows)
		++row;
	    else
		scroll(0,0,rows,cols,1);
	    break;
	case CR:  /* Carriage return */
	    col = 0;
	    break;
      }
    else
      {	/* Show character with attributes. Writes directly to Video RAM */
	    if (col <= cols)		/* Only display if on screen */
	      {	v = vid_mem;
		v += (bpr * row) + (col << 1);
		*v++ = c;
		*v   = cur_attr;
		col++;
	      }
      }
}

/* wchar --------------------------------------------------------
   Writes a character to screen.
   Moves the cursor.
*/

void wchar (unsigned char c)
{
	_wchar(c);
	setcur(row,col);
}

/* wstr --------------------------------------------------------
   Writes a string to screen */

void wstr(unsigned char *str)
{
   while (*str)
	_wchar(*str++);			/* _wchar DOES NOT!! move the cursor */
   setcur(row,col);			/* Move the cursor */
}

/* wstr0 -------------------------------------------------------- */
/* This version doesn't move the cursor: */
void wstr0(unsigned char *str)
{  register short posn;

   posn=getcur();
   while (*str)
	_wchar(*str++);			/* _wchar DOES NOT!! move the cursor */

   row = posn>>8;			/* reset cursor variables */
   col = posn&255;
}

unsigned int rchar (int row, int col)
{ int retval=0;
  unsigned char far *v;

	v = vid_mem;
	v += (bpr * row) + (col << 1);
	retval = *v++;
	retval |= (int)(*v) << 8; /* Attribute in high byte */
	return(retval);
}

/* erase -------------------------------------------------------- */
/*
	Erase a region of the screen.
	Does not move the cursor.
	If -1 is specified as the minimum row, the
	current cursor line will be used.
*/
void erase (int min_row, int min_col, int max_row, int max_col)
{   int posn;

    posn=getcur();			/* Save current posn */

    if (min_row < 0)
      min_row = posn>>8;		/* Use current cursor line */
    if (max_row < 0)
      max_row = posn>>8;
    if (min_col < 0)
      min_col = posn&255;		/* Use current cursor column */
    if (max_col < 0)
      max_col = posn&255;

    for (row=min_row; row <= max_row; row++)
      for (col=min_col; col <= max_col; )
	_wchar(32);

    row = posn>>8;			/* reset row & col */
    col = posn&255;
}

/* clear_eol --------------------------------------------------
   Clears to the end of the line */

void clear_eol()
{  register short posn;

	erase(row,col,row,cols);	/* Write spaces to end of line */
}

/*---------------------------------- scroll ---------------------
    Scrolls up/down the given number of lines from top line
*/
void scroll(int top_row,int left_col,int bot_row,int right_col,int nlines)
{
    if (nlines>=0)
	Regs.h.ah=SCROLL_UP;
    else
      { Regs.h.ah=SCROLL_DOWN;
	nlines = -nlines;
      }
    Regs.h.al=nlines;
    Regs.h.ch=top_row;
    Regs.h.cl=left_col;
    Regs.h.dh=bot_row;
    Regs.h.dl=right_col;
    Regs.h.bh=color_attribs[1];
    int86(VIDEO_INT, &Regs, &Regs);
}

/* put_box_char --------------------------------------------------- */
/*

	Put a character on the screen at a given place.
*/

void put_box_char(int row,int col,unsigned char ch)
{	setcur(row,col);
	wchar(ch);
}

/* box ------------------------------------------------------------- */
/*
	Draws a box on the screen.
	dflag tells us whether to draw a single (1) or double (2) box.
*/

void box(int y1, int x1, int y2, int x2, int dflag)
{
    register int temp;
    register int edge;
    char aline[82];
    static unsigned char corners [4] [6] = 
			      { {218, 192, 191, 217, 196, 179}, /* single */
				{201, 200, 187, 188, 205, 186}}; /* double */

	dflag--;	/* Array is 0-based */
	edge = 0;	/* assume not box, hence no edge */
	if ((y1 != y2) && (x1 != x2))	/* full-fledged Box! */
	    {	put_box_char(y1,x1,corners[dflag][0]);
		put_box_char(y2,x1,corners[dflag][1]);
		put_box_char(y1,x2,corners[dflag][2]);
		put_box_char(y2,x2,corners[dflag][3]);
		++edge;			/* lines drawn INSIDE box */
	    }
	if (x1 != x2)
	    {	temp = x2-x1+1-edge*2;
		memset(aline,corners[dflag][4],temp);
		aline[temp]='\0';
		setcur(y1,x1+edge);
		wstr(aline);
		setcur(y2,x1+edge);
		wstr(aline);
	    }
	if (y1 != y2)
	  for ( temp=y1+edge; temp<=y2-edge; ++temp)
	    {	put_box_char(temp,x1,corners[dflag][5]);
		put_box_char(temp,x2,corners[dflag][5]);
	    }
	setcur(y1+1,x1+2);
}
