/*

	SETEGAD.H -- Set EGAD Definitions

	EGAD

	Copyright (c) 1991 LS Software

	8139 E. Mawson
	Mesa, AZ 85207
	(602) 380-9175

	All Rights Reserved. No part of this program may be reproduced
	in whole or in part without express permission of LS Software

*/

/* BIOS variables */

#define	PRTSC_STAT 0x00400100L	/* BIOS Print Screen Status */
#define	SERPORTS 0x00400000L	/* BIOS serial port table */
#define	PARPORTS 0x00400008L	/* BIOS parallel port table */
#define MAXPORT 4		/* COM4: is last number */

#define EGAD_IDENT 0X5A51		/* EGAD identity flag */

#define JUST_LEFT	1	/* Left justified */
#define JUST_RIGHT	2	/* Right justified */
#define JUST_CENTER	3	/* Center printouts left-right */

#define VJUST_TOP	1
#define VJUST_BOTTOM	2
#define VJUST_CENTER	3

#define ORIENT_AUTO	0
#define ORIENT_PORT	1
#define ORIENT_LAND	2

#define TSRF_REVCOLOR	0x80

/* New-style configuration block, introduced EGAD 2.35: */

typedef struct
{   unsigned int identity_flag;
    unsigned int printer_flag;
    unsigned int signature;		/* structure size */
    unsigned int palette_2[2];		/* palettes */
    unsigned int palette_4[4];
    unsigned int palette_16[16];
    unsigned int palette_text[16];
    unsigned int tsr_x1,tsr_y1,tsr_x2,tsr_y2;	/* bounding box */
    unsigned int setff;		/* 0=no ff, 1=ff after print, 2=ff only */
    signed   int unit;		/* printer unit ( <0 for serial, >0 parallel) */
    unsigned int tsr_enlarge;	/* enlargement 0..4 */
    unsigned int vga_divisor;	/* divisor for 256-color modes */
    unsigned int tsr_box;	/* 1 if want crop box to come up */
    unsigned int centering;	/* JUST_* from above */
    unsigned char vcentering;	/* VJUST_* from above */
    unsigned char tsrflags;	/* TSRF_* bits from above */
    unsigned int orientation;	/* ORIENT_* from above */
    unsigned int crop_colora;	/* Color of currently-adjusted crop box corner */
    unsigned int crop_colorb;	/* Color of 'dead' corner of crop box */
    unsigned int crop_incr;	/* increment when adjusting crop box */

    unsigned int hdr_nvars;	/* Number of data WORDS (int's) which follow */
    /* NOTE!!! The format of the header was changed in EGAD Version 2.41. */
    /* If the header version is less than 241, do not allow access to any */
    /* of the below variables!!! -- wl 12/18/92 */
    unsigned int hdr_version;
    unsigned int tsr_psp_seg;	/* PSP of a TSR copy of EGAD */

    unsigned char hdr_nmap;	/* # of screen-to-printer maps (1,2,..) */
    unsigned char hdr_ntextatts; /* Number of text attributes (e.g., 4) */
    unsigned int hdr_map0;	/* Offset to first map */
    unsigned char hdr_secmap;	/* mode #, first alternate map (e.g., 6) */
    unsigned char hdr_pixorient; /* 0=vertical [matrix], 1=horizontal [laser] */
    unsigned char hdr_nmodes;	/* # of printer graphics modes (1,2,..) */
    unsigned char hdr_resrv0;	/* (reserved) */
    unsigned int hdr_mode0;	/* Offset to first graphics mode */
    unsigned int hdr_pat0;	/* Offset to first pattern */
    unsigned char hdr_dith_n;	/* # of dither tables */
    unsigned char hdr_iscolor;	/* Nonzero if a color printer */
    unsigned char hdr_dith_lev;	/* Levels per table */
    unsigned char hdr_ncolors;	/* # of color buffers */
				/* NOTE: hdr_ncolors=0 if hdr_version<242 */
				/* Number of actual printer graphics colors is: */
				/*   hdr_ncolors * hdr_dith_lev */
    unsigned char hdr_dith_wid;	/* Bytes per level */
    unsigned char hdr_resrv1;	/* (reserved) */
    unsigned int hdr_dith0;	/* Offset to first dither table */

} PAR_T;

/* Size of structure block.  Not tied to the ENTIRE above structure, but */
/* rather only the part in the CONFIG_DATA structure (see GLOBAL.INC in  */
/* the EGAD source code.) */
#define EGAD_STRUC_SIZE 30

typedef struct
{   unsigned int identity_flag;
    unsigned int printer_flag;
    unsigned int setff;
    unsigned int unit;
    unsigned int palette_2[2];
    unsigned int palette_4[4];
    unsigned int palette_16[16];
    unsigned int palette_text[16];
    unsigned int tsr_x1,tsr_y1,tsr_x2,tsr_y2;
    unsigned int tsr_box;
    unsigned int tsr_enlarge;
    unsigned int vga_divisor;
} OLD_PAR_T;

#define P_TEXT_NONE	0	/* Printer does B&W text only */
#define P_TEXT_ATTRS	1	/* Printer does attributes in text mode */
#define P_TEXT_COLOR	2	/* Printer does foreground colors */
#define P_TEXT_BG_COLOR	4	/* Printer does background colors */

#define P_COLORS_ONLY  0	/* Only values in printer_palette[] are legal */
#define P_COLORS_OTHER 1	/* Other values possible ("See color chart") */

#define P_TYPE_COLOR   1	/* Color capable */
#define P_TYPE_MATRIX  2	/* Dot matrix */
#define P_TYPE_INKJET  4	/* Ink jet */
#define P_TYPE_LASER   8	/* Laser */
#define P_TYPE_MAT24  16	/* 24-pin matrix */

typedef struct
{   int signature;		/* 1-Word printer signature */
    char *Printer_Name;		/* Name of printer */
    char *Program_Name;		/* Default EGAD??.EXE for printer */
    int other_colors;		/* P_COLORS_* from above */
    int printer_text_mode;	/* P_TEXT_* from above */
    PAR_T *default_palette;	/* Default palette table */
    int *(printer_palette[2]);	/* Pointer to printer palette arrays */
} PRINTER_DEF;

/* --------------------------
 * Within the SIGNATURE word:
 * --------------------------
 */

#define TYP_COLOR	0x0200		/* Color graphics */
#define TYP_CTEXT	0x0400		/* Color text */
#define TYP_LASER	0x0800		/* Lasers, incl. HP and PostScript */
#define TYP_MATRIX	0x1000		/* all matrix printers */
#define TYP_MAT24	0x3000		/* 24-pin matrix only */
#define TYP_INKJET	0x4000		/* Inkjets not otherwise compatible */

/* Interface Routines */

extern void center_text(char *text, int line);
extern void instruction(char *inst_text, int y1, int x1);
extern void erase_instruction();
extern void show_error(char *the_error, int line, int col, int bad_err);
extern int menu(char *menu_title,char *menu_items[],int menu_item_count,
	 int toprow, int leftcol);
extern int get_int(int init_val, int exit_on_arrows);
extern void highlight(char *text, int row, int col, int mode);
