/* 
	ZCON.C

	Keyboard input and screen output for Z-100 computer.
	Compatible with Microsoft C Version 5.0.

	Copyright (c) 1990 Lindley Systems

	4257 Berwick Place
	Woodbridge, VA 22192-5119
	(703) 590-8890

*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include "keys.h"

/* BDOS call 7 doesn't check for ^C's */

#define BDOS_IN      7

/* Prototypes */
void init_keys();
void exit_keys();
int  gtkey();
void wchar (unsigned char c);
void wstr(unsigned char *str);
void wstr0(unsigned char *str);
void color(int color_byte);
void box(int y1, int x1, int y2, int x2, int dflag);
void put_box_char(int row,int col,unsigned char ch);
void clear_eol();
void erase (int min_row, int min_col, int max_row, int max_col);
void cls ();
void setcur(int to_row, int to_col) ;
int  getcur();
void cursize (int kind);
int  getpage();
int  video_mode();
int  set_rows(int num_rows);

extern void conout(unsigned char c);

/*
	Set NUMPAD to 1 if you want to map number pad entries to
	numeric keys, or 0 to pass them through.
*/
#define NUMPAD 1

static char far *vid_mem = (char far *)0xE0000000L;
static int crsr_x=0;	/* Need to keep this, since clear_eol if we're on the
			   80th column already should do **NOTHING** */

int rows=24;
int cols=79;

/* Color attributes */
int cur_attr;
int color_attribs[4]={0x70,0x07,0x70,0x07};

int  system_type()
{	return(1);		/* Indicate Z-100 */
}

/* gtkey ------------------------------------------------------
   Return a key from the keyboard and map control characters */

int gtkey()
{
    register int c;

    c=bdos(BDOS_IN,0,0) & 0xff;
#if NUMPAD
    if (c >= 0xAD && c <= 0xB9)	/* Keypad? Map to numbers & symbols */
	return(c & 0x3F);
#endif
    switch(c)
    {
    case 0x7F:
        return(BS);
    case 0x8D:
    case 0xCD:
        return('\n');
    case 0xE4:
        return(0x7f);
    case 0x95:		/* HELP key, make same as F1 */
	return(0x97);
    case '\r':
        return ('\n');
    case '\n':
        return ('\r');
    default:
        return(c);
    }
}

/* save cursor position */
static void savecursor()
{	conout('\033');
	conout('j');
}

static void restorecursor()
{	conout('\033');
	conout('k');
} 

/*	color (fore, back)

	Sets the current color.

	color_byte = 0bbbffff
			 ^^^^-- 0..15, Foreground
		      ^^^------ 0...7, Background
*/

/* This color map actually works both ways - from Z-100 to PC or back. */
static int color_map[8]={0,1,4,5,2,3,6,7};

void color(int color_byte)
{
	cur_attr = color_byte;
	conout('\033');
	conout('m');
	conout('0'+(color_map[color_byte&0xF]));
	conout('0'+(color_map[color_byte>>4]));
}

static char *grafmap =
"iii`tttcct`cdddceusvabvvefusvabuusseeffbbdf\000PQqp                 g    j ^^^   ^ ";
		/* the null, P, and Q will print out as inverse
		      space, p, and q, respectively */

/* wchar -------------------------------------------------------
   Writes one character to screen */

void wchar(unsigned char c)
{
    static int grafmode=0;		/* in the graphics mode? */

    c &= 255;
    switch (c)
    {
        case '\01':
        case '\02':
        case '\03':
        case '\04':
	    color(color_attribs[c-1]);
            break;
        default:
	    crsr_x++;			/* don't worry about wrapping */
	    if (c<127)
	    {	if (grafmode)		/* restore normal mode */
		{   conout ('\033');
		    conout ('G');
		    grafmode=0;
		}
		conout(c);
		if (c == 13)		/* Carriage return */
		  crsr_x = 0;
	    }
	    else
	    {   if (c>=0xB0)		/* if in the defined special chars */
		{
		    if (!grafmode)	/* put us into the graphics mode */
		    {   conout ('\033');
			conout ('F');
			grafmode++;
		    }
		    c = grafmap[c-0xB0];
		    if ((c < '^') && (c != 32))
			/* Anything but Space or Lowercase -> Inverse */
		    {	conout('\033');conout('p');
			conout(c+32);
			conout('\033');conout('q');
		    }
		    else conout(c);
		}
		else conout(32);	/* not implemented: print a space */
	    }
    }
}

/* wstr --------------------------------------------------------
   Writes a string to screen */

void wstr(unsigned char *str)
{
	while (*str) wchar(*str++);
}

void wstr0(unsigned char *str)
{
	savecursor();
	while (*str) wchar(*str++);
	restorecursor();
}

unsigned int rchar (unsigned int row, unsigned int col)
{ int retval=0, attr;
  unsigned char far *v;
  unsigned char c, *gp;

	v = vid_mem;
	v += (row * 0x800) + col + 0x480;/* 9th scan line of char pos'n */

	c = (*v) + 0x20;		/* Character from screen */

	if (c >= 0x80)			/* Graphics character? */
	  { c -= 0x22;			/* 0x60->0x5E; 0x63->0x61 */
	    for (retval=0,gp=grafmap; retval<80; retval++, gp++)
	      if (c == *gp)
	        { c = retval + 0xB0;
	          break;
	        }
	  }

	attr = (int)*(v + 0x80);		/* Get attribute and convert */
						/* Z-100 color to PC color */
	retval = color_map[attr & 0x07];	/* Low nybble */
	retval |= color_map[(attr >> 4) & 0x07] << 4;	/* High nybble */

	return(retval << 8 | c);
	/* Return PC style attribute in high byte, and character
	   in low byte */
}

/* cls ---------------------------------------------------------
   Clears the console screen */

void cls ()

{
	/* Note -- we have to be on the main screen, not the 25th line!! */
    wstr("\033Y8 \033K\033H\033E");
    crsr_x = 0;
}

int set_rows(int num_rows)
{
	return(25);
}

/* key_delay -----------------------------

	Delay a given number of 100ths of seconds, up to 60, or
	until a key is hit.

*/

static int getclk()
{
  struct dostime_t time;

  _dos_gettime(&time);
  return((int)(time.second)*100+time.hsecond);
}

void key_delay (int nhunds)
{
  register int waitfor;

  waitfor = getclk()+nhunds;
  if (waitfor >= 6000)	/* Overflowed seconds into minutes */
    waitfor -= 6000;
	/* If we went 'backwards' to the beginning of the next minute,
	   wait for this minute to end... */
  while (getclk() > waitfor && !kbhit())
	;
	/* Now wait until proper moment in the current second */
  while (getclk() < waitfor && !kbhit())
	;
}

/* init_keys ---------------------------------------------------------
   Initializes the keyboard to the 8 bit mode */

void init_keys()

{ int got_esc=0;

    printf("\033Z");	/* Tell it to identify terminal type */

    key_delay(10);	/* Delay 10/100 = 1/10 of a second */

    while (kbhit())	/* Read the whole input buffer */
      if (getch() == 27)
        got_esc++;		/* Found an ESC char, probably a Z-100 */

    if (got_esc == 0)		/* Whoa!! Must not be Z-100 */
      { fprintf(stderr,"Unknown Video Mode.");
        exit(9);
      }

/*
	Reset terminal; don't wrap lines; disable key expansion;
	block cursor; 25th line; send 'Normal Video' color sequence.
*/
    wstr("\033z\033w\033y?\033y4\033x1\002");
    setcur(0,0);		/* Reset X,Y variables */
    outp(0xD8,0x78);		/* Allow access to all planes */

}
/* -------------------- exit_keys -----------------------------

	Restores the keyboard to the normal mode

	Exit graphics mode, enable key expansion, underline cursor,
	white-on-black, disable 25th line, and move to last line of display.
*/

void exit_keys()

{
	wstr("\033G\033x?\033y4\033m70\033y1\033Y7 ");
}

/* cursize -----------------------------------------------------
   sets the height of the cursor */

void cursize (int kind)
{
    wchar(27);
    wchar(kind==0?'x':'y');	/* Disable cursor if kind == 0 */
    wchar('5');
    wchar(27);
    wchar(kind==1?'y':'x');	/* kind=1 for underline, 2 for block */
    wchar('4');
}

/* setcur ------------------------------------------------------
   Sets the cursor to the given position */

void setcur(int to_row, int to_col) 
{
    conout(27); conout('Y');	/* cursor addressing */
    conout(to_row+32); conout(to_col+32);
    crsr_x = to_col;
}

/* clear_eol --------------------------------------------------
   Clears to the end of the line */

void clear_eol()
{
	if (crsr_x < 79)		/* Do nothing if on 80th character */
	  { conout(27);
	    conout('K');		/* Clear to end of line */
	  }
}

/*	Erase a part of the screen.  Does not move the cursor.
	If -1 is passed as min_row and max_row, the current line will
	be used.  (setcur's escape code will have an invalid line #,
	which the Z-100 will ignore.)
*/
void erase (int min_row, int min_col, int max_row, int max_col)
{   int i,j;

    savecursor();
   
    for (i=min_row; i <= max_row; i++)
      {	if (max_col == 79)	/* Erase to end of line? */
	  { setcur(i,min_col);
	    clear_eol();
	  }
	else
	  if (min_col == 0)	/* Erase to start of line? */
	    { setcur(i,max_col);
	      conout(27); conout('o');
	    }
	  else			/* Erase just part of line */
	    { setcur(i,min_col);
	      for (j=min_col; j <= max_col; j++)
	        wchar(32);
	    }
      }
    restorecursor();
}

/*
	put_box_char

	Put a character on the screen at a given place.
*/

void put_box_char(int row,int col,unsigned char ch)
{	setcur(row,col);
	wchar(ch);
}

/*
	Draws a box on the screen.
	dflag tells us whether to draw a single (1) or double (2) box.
*/

void box(int y1, int x1, int y2, int x2, int dflag)
{
    register int temp;
    register int edge;
    unsigned char aline[82];
    static unsigned char corners [4] [6] = 
			      { {218, 192, 191, 217, 196, 179}, /* single */
				{201, 200, 187, 188, 205, 186}}; /* double */

	dflag--;	/* Array is 0-based */
	edge = 0;	/* assume not box, hence no edge */
	if ((y1 != y2) && (x1 != x2))	/* full-fledged Box! */
	    {	put_box_char(y1,x1,corners[dflag][0]);
		put_box_char(y2,x1,corners[dflag][1]);
		put_box_char(y1,x2,corners[dflag][2]);
		put_box_char(y2,x2,corners[dflag][3]);
		++edge;			/* lines drawn INSIDE box */
	    }
	if (x1 != x2)
	    {	temp = x2-x1+1-edge*2;
		memset(aline,corners[dflag][4],temp);
		aline[temp]='\0';
		setcur(y1,x1+edge);
		wstr(aline);
		setcur(y2,x1+edge);
		wstr(aline);
	    }
	if (y1 != y2)
	  for ( temp=y1+edge; temp<=y2-edge; ++temp)
	    {	put_box_char(temp,x1,corners[dflag][5]);
		put_box_char(temp,x2,corners[dflag][5]);
	    }
	setcur(y1+1,x1+2);
}

int  video_mode()
{
	return(3);
}

